@@ -1,11 +1,12 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
 import gettext
 
-from tryton.common import get_toplevel_window, IconFactory
+from gi.repository import Gtk
+
+from tryton.common import (get_toplevel_window, IconFactory,
+    timezoned_date, untimezoned_date)
 from tryton.common.datetime_ import date_parse
-from tryton.common.datetime_strftime import datetime_strftime
 from tryton.common.underline import set_underline
 from tryton.config import TRYTON_ICON
 from tryton.gui import Main
@@ -18,31 +19,34 @@ class Revision(object):
 
     def __init__(self, revisions, revision=None, format_='%x %H:%M:%S.%f'):
         self.parent = get_toplevel_window()
-        self.win = gtk.Dialog(_('Revision'), self.parent,
-            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
+        self.win = Gtk.Dialog(
+            title=_('Revision'), transient_for=self.parent, modal=True,
+            destroy_with_parent=True)
         Main().add_window(self.win)
         cancel_button = self.win.add_button(
-            set_underline(_("Cancel")), gtk.RESPONSE_CANCEL)
+            set_underline(_("Cancel")), Gtk.ResponseType.CANCEL)
         cancel_button.set_image(IconFactory.get_image(
-                'tryton-cancel', gtk.ICON_SIZE_BUTTON))
+                'tryton-cancel', Gtk.IconSize.BUTTON))
         cancel_button.set_always_show_image(True)
         ok_button = self.win.add_button(
-            set_underline(_("OK")), gtk.RESPONSE_OK)
+            set_underline(_("OK")), Gtk.ResponseType.OK)
         ok_button.set_image(IconFactory.get_image(
-                'tryton-ok', gtk.ICON_SIZE_BUTTON))
+                'tryton-ok', Gtk.IconSize.BUTTON))
         ok_button.set_always_show_image(True)
-        self.win.set_default_response(gtk.RESPONSE_OK)
+        self.win.set_default_response(Gtk.ResponseType.OK)
         self.win.set_icon(TRYTON_ICON)
         self.win.vbox.set_spacing(3)
-        self.win.vbox.pack_start(gtk.Label(
-                _('Select a revision')), expand=False, fill=True)
-        self.win.vbox.pack_start(gtk.HSeparator())
-        hbox = gtk.HBox(spacing=3)
-        label = gtk.Label(_('Revision:'))
-        hbox.pack_start(label, expand=True, fill=True)
-        list_store = gtk.ListStore(str, str)
+        self.win.vbox.pack_start(Gtk.Label(
+                label=_('Select a revision')),
+            expand=False, fill=True, padding=0)
+        self.win.vbox.pack_start(
+            Gtk.HSeparator(), expand=True, fill=True, padding=0)
+        hbox = Gtk.HBox(spacing=3)
+        label = Gtk.Label(label=_('Revision:'))
+        hbox.pack_start(label, expand=True, fill=True, padding=0)
+        list_store = Gtk.ListStore(str, str)
         # Set model on instantiation to get the default cellrenderer as text
-        combobox = gtk.ComboBoxEntry(model=list_store)
+        combobox = Gtk.ComboBox(model=list_store, has_entry=True)
         self.entry = combobox.get_child()
         self.entry.connect('focus-out-event', self.focus_out)
         self.entry.connect('activate', self.activate)
@@ -51,7 +55,7 @@ class Revision(object):
         self.entry.set_property('activates_default', True)
         self._format = format_
         if revision:
-            self.entry.set_text(datetime_strftime(revision, self._format))
+            self.entry.set_text(revision.strftime(self._format))
             self._value = revision
             active = -1
         else:
@@ -59,16 +63,17 @@ class Revision(object):
             active = 0
         list_store.append(('', ''))
         for i, (rev, id_, name) in enumerate(revisions, 1):
-            list_store.append((datetime_strftime(rev, self._format), name))
+            list_store.append(
+                (timezoned_date(rev).strftime(self._format), name))
             if rev == revision:
                 active = i
         combobox.set_active(active)
-        cell = gtk.CellRendererText()
-        combobox.pack_start(cell, True)
+        cell = Gtk.CellRendererText()
+        combobox.pack_start(cell, expand=True)
         combobox.add_attribute(cell, 'text', 1)
-        hbox.pack_start(combobox, expand=True, fill=True)
+        hbox.pack_start(combobox, expand=True, fill=True, padding=0)
         combobox.set_entry_text_column(0)
-        self.win.vbox.pack_start(hbox, expand=True, fill=True)
+        self.win.vbox.pack_start(hbox, expand=True, fill=True, padding=0)
         self.win.show_all()
 
     def focus_out(self, entry, event):
@@ -94,7 +99,7 @@ class Revision(object):
         value = None
         if text:
             try:
-                value = date_parse(text, self._format)
+                value = untimezoned_date(date_parse(text, self._format))
             except ValueError:
                 pass
         self._value = value
@@ -103,12 +108,12 @@ class Revision(object):
         if not self._value:
             self.entry.set_text('')
         else:
-            self.entry.set_text(datetime_strftime(self._value, self._format))
+            self.entry.set_text(self._value.strftime(self._format))
 
     def run(self):
         response = self.win.run()
         revision = None
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             revision = self._value
         self.parent.present()
         self.win.destroy()
