@@ -5,15 +5,23 @@ import datetime
 import goocalendar
 from .dates_period import DatesPeriod
 
+from tryton.common import MODELACCESS
+from tryton.config import CONFIG
+
+_colors = CONFIG['calendar.colors'].split(',')
+
 
 class Calendar_(goocalendar.Calendar):
     'Calendar'
 
-    def __init__(self, attrs, screen, fields, event_store=None):
+    def __init__(self, attrs, view, fields, event_store=None):
         super(Calendar_, self).__init__(
             event_store, attrs.get('mode', 'month'))
+        self.props.selected_border_color = _colors[1]
+        if hasattr(self.props, 'selected_text_color'):
+            self.props.selected_text_color = _colors[0]
         self.attrs = attrs
-        self.screen = screen
+        self.view_calendar = view
         self.fields = fields
         self.event_store = event_store
         self.current_domain_period = self.get_displayed_period()
@@ -27,6 +35,9 @@ class Calendar_(goocalendar.Calendar):
 
     def get_displayed_period(self):
         cal = calendar.Calendar(self.firstweekday)
+        if self.view == 'day':
+            first_date = self.selected_date
+            last_date = self.selected_date + datetime.timedelta(1)
         if self.view == 'week':
             week = goocalendar.util.my_weekdatescalendar(cal,
                 self.selected_date)
@@ -64,10 +75,10 @@ class Calendar_(goocalendar.Calendar):
         return domain
 
     def get_colors(self, record):
-        text_color = None
+        text_color = _colors[0]
         if self.attrs.get('color'):
             text_color = record[self.attrs['color']].get(record)
-        bg_color = 'lightblue'
+        bg_color = _colors[1]
         if self.attrs.get('background_color'):
             bg_color = record[self.attrs['background_color']].get(
                 record)
@@ -79,25 +90,28 @@ class Calendar_(goocalendar.Calendar):
                 and not isinstance(value, datetime.datetime))
         dtstart = self.attrs['dtstart']
         dtend = self.attrs.get('dtend')
-        if self.screen.current_record:
-            record = self.screen.current_record
+        if self.view_calendar.record:
+            record = self.view_calendar.record
             date = record[dtstart].get(record)
             if date:  # select the day of the current record
                 self.select(date)
 
-        if self._event_store:
-            self._event_store.clear()
-        else:
-            event_store = goocalendar.EventStore()
-            self.event_store = event_store
+        event_store = goocalendar.EventStore()
+
+        model_access = MODELACCESS[self.view_calendar.screen.model_name]
+        editable = (
+            bool(int(self.view_calendar.attributes.get('editable', 1)))
+            and model_access['write'])
 
         for record in group:
             if not record[dtstart].get(record):
                 continue
 
             start = record[dtstart].get_client(record)
+            record[dtstart].state_set(record)
             if dtend:
                 end = record[dtend].get_client(record)
+                record[dtend].state_set(record)
             else:
                 end = None
             midnight = datetime.time(0)
@@ -114,9 +128,17 @@ class Calendar_(goocalendar.Calendar):
             text_color, bg_color = self.get_colors(record)
             label = '\n'.join(record[attrs['name']].get_client(record)
                 for attrs in self.fields).rstrip()
+            event_editable = (
+                editable
+                and not record[dtstart].get_state_attrs(record).get(
+                    'readonly', False)
+                and (not dtend
+                    or not record[dtend].get_state_attrs(record).get(
+                        'readonly', False)))
             event = goocalendar.Event(label, start, end, text_color=text_color,
-                bg_color=bg_color, all_day=all_day)
+                bg_color=bg_color, all_day=all_day, editable=event_editable)
             event.record = record
-            self._event_store.add(event)
+            event_store.add(event)
+        self.event_store = event_store
 
         self.grab_focus(self.get_root_item())
