@@ -14,6 +14,7 @@ from tryton.gui.window.win_form import WinForm
 from tryton.gui.window.view_form.screen import Screen
 from tryton.common import file_selection, file_open, file_write
 import tryton.common as common
+from tryton.common import get_gdk_backend
 from tryton.common.cellrendererbutton import CellRendererButton
 from tryton.common.cellrenderertext import CellRendererText, \
     CellRendererTextCompletion
@@ -1076,10 +1077,12 @@ class Selection(GenericText, SelectionMixin, PopdownMixin):
     def editing_started(self, cell, editable, path):
         super(Selection, self).editing_started(cell, editable, path)
         record, field = self._get_record_field_from_path(path)
-        # Combobox does not emit remove-widget when focus is changed
-        self.editable.connect(
-            'editing-done',
-            lambda *a: self.editable.emit('remove-widget'))
+        gdk_backend = get_gdk_backend()
+        if gdk_backend == 'x11':
+            # Combobox does not emit remove-widget when focus is changed
+            self.editable.connect(
+                'editing-done',
+                lambda *a: self.editable.emit('remove-widget'))
 
         selection_shortcuts(editable)
 
@@ -1087,6 +1090,14 @@ class Selection(GenericText, SelectionMixin, PopdownMixin):
             return self.set_value(editable, record, field)
         editable.get_child().connect('activate', set_value)
         editable.get_child().connect('focus-out-event', set_value)
+
+        if gdk_backend != 'x11':
+            # For non X11 backend we can rely the focus-out-event to be sent
+            # correctly
+            def remove_entry(entry, event):
+                editable.emit('editing-done')
+                editable.emit('remove-widget')
+            editable.get_child().connect('focus-out-event', remove_entry)
         editable.connect('changed', set_value)
 
         self.update_selection(record, field)
@@ -1290,7 +1301,7 @@ class Button(Cell):
         states = record.expr_eval(self.attrs.get('states', {}))
         invisible = states.get('invisible', False)
         cell.set_property('visible', not invisible)
-        readonly = states.get('readonly', False)
+        readonly = record.readonly or states.get('readonly', False)
         cell.set_property('sensitive', not readonly)
         parent = record.parent if record else None
         while parent:
