@@ -294,7 +294,7 @@ class WizardForm(Wizard, TabContent, SignalEvent):
 
     def set_cursor(self):
         if self.screen:
-            self.screen.set_cursor()
+            self.screen.set_cursor(reset_view=False)
 
 
 class WizardDialog(Wizard, NoModal):
