@@ -1,5 +1,6 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+import copy
 import xmlrpc.client
 import json
 import ssl
@@ -11,7 +12,9 @@ import hashlib
 import base64
 import threading
 import errno
+import logging
 from functools import partial
+from collections import defaultdict
 from contextlib import contextmanager
 from functools import reduce
 from urllib.parse import urljoin, quote
@@ -20,6 +23,7 @@ __all__ = ["ResponseError", "Fault", "ProtocolError", "Transport",
     "ServerProxy", "ServerPool"]
 CONNECT_TIMEOUT = 5
 DEFAULT_TIMEOUT = None
+logger = logging.getLogger(__name__)
 
 
 class ResponseError(xmlrpc.client.ResponseError):
@@ -32,11 +36,8 @@ class Fault(xmlrpc.client.Fault):
         super(Fault, self).__init__(faultCode, faultString, **extra)
         self.args = faultString
 
-    def __repr__(self):
-        return (
-            "<Fault %s: %s>" %
-            (repr(self.faultCode), repr(self.faultString))
-            )
+    def __str__(self):
+        return str(self.faultCode)
 
 
 class ProtocolError(xmlrpc.client.ProtocolError):
@@ -131,7 +132,8 @@ class Transport(xmlrpc.client.SafeTransport):
     accept_gzip_encoding = True
     encode_threshold = 1400  # common MTU
 
-    def __init__(self, fingerprints=None, ca_certs=None, session=None):
+    def __init__(
+            self, fingerprints=None, ca_certs=None, session=None):
         xmlrpc.client.Transport.__init__(self)
         self._connection = (None, None)
         self.__fingerprints = fingerprints
@@ -143,6 +145,18 @@ class Transport(xmlrpc.client.SafeTransport):
         parser = JSONParser(target)
         return parser, target
 
+    def parse_response(self, response):
+        cache = None
+        if hasattr(response, 'getheader'):
+            cache = int(response.getheader('X-Tryton-Cache', 0))
+        response = super().parse_response(response)
+        if cache:
+            try:
+                response['cache'] = int(cache)
+            except ValueError:
+                pass
+        return response
+
     def get_host_info(self, host):
         host, extra_headers, x509 = xmlrpc.client.Transport.get_host_info(
             self, host)
@@ -167,23 +181,12 @@ class Transport(xmlrpc.client.SafeTransport):
     def make_connection(self, host):
         if self._connection and host == self._connection[0]:
             return self._connection[1]
-        host, self._extra_headers, x509 = self.get_host_info(host)
+        chost, self._extra_headers, x509 = self.get_host_info(host)
 
         ssl_ctx = ssl.create_default_context(cafile=self.__ca_certs)
 
-        class HTTPSConnection(http.client.HTTPSConnection):
-
-            def connect(self):
-                sock = socket.create_connection((self.host, self.port),
-                    self.timeout)
-                if self._tunnel_host:
-                    self.sock = sock
-                    self._tunnel()
-                self.sock = ssl_ctx.wrap_socket(
-                    sock, server_hostname=self.host)
-
         def http_connection():
-            self._connection = host, http.client.HTTPConnection(host,
+            self._connection = host, http.client.HTTPConnection(chost,
                 timeout=CONNECT_TIMEOUT)
             self._connection[1].connect()
             sock = self._connection[1].sock
@@ -191,8 +194,8 @@ class Transport(xmlrpc.client.SafeTransport):
             sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
 
         def https_connection(allow_http=False):
-            self._connection = host, HTTPSConnection(host,
-                timeout=CONNECT_TIMEOUT)
+            self._connection = host, http.client.HTTPSConnection(chost,
+                timeout=CONNECT_TIMEOUT, context=ssl_ctx)
             try:
                 self._connection[1].connect()
                 sock = self._connection[1].sock
@@ -204,19 +207,20 @@ class Transport(xmlrpc.client.SafeTransport):
                     peercert = None
 
                 def format_hash(value):
-                    return reduce(lambda x, y: x + y[1].upper() +
-                        ((y[0] % 2 and y[0] + 1 < len(value)) and ':' or ''),
+                    return reduce(lambda x, y: x + y[1].upper()
+                        + ((y[0] % 2 and y[0] + 1 < len(value)) and ':' or ''),
                         enumerate(value), '')
                 return format_hash(hashlib.sha1(peercert).hexdigest())
-            except ssl.SSLError:
+            except (socket.error, ssl.SSLError, ssl.CertificateError):
                 if allow_http:
                     http_connection()
                 else:
                     raise
 
         fingerprint = ''
-        if self.__fingerprints is not None and host in self.__fingerprints:
-            if self.__fingerprints[host]:
+        if (self.__fingerprints is not None
+                and self.__fingerprints.exists(chost)):
+            if self.__fingerprints.get(chost):
                 fingerprint = https_connection()
             else:
                 http_connection()
@@ -224,7 +228,7 @@ class Transport(xmlrpc.client.SafeTransport):
             fingerprint = https_connection(allow_http=True)
 
         if self.__fingerprints is not None:
-            self.__fingerprints[host] = fingerprint
+            self.__fingerprints.set(chost, fingerprint)
         self._connection[1].timeout = DEFAULT_TIMEOUT
         self._connection[1].sock.settimeout(DEFAULT_TIMEOUT)
         return self._connection[1]
@@ -234,7 +238,7 @@ class ServerProxy(xmlrpc.client.ServerProxy):
     __id = 0
 
     def __init__(self, host, port, database='', verbose=0,
-            fingerprints=None, ca_certs=None, session=None):
+            fingerprints=None, ca_certs=None, session=None, cache=None):
         self.__host = '%s:%s' % (host, port)
         if database:
             database = quote(database)
@@ -243,15 +247,22 @@ class ServerProxy(xmlrpc.client.ServerProxy):
             self.__handler = '/'
         self.__transport = Transport(fingerprints, ca_certs, session)
         self.__verbose = verbose
+        self.__cache = cache
 
     def __request(self, methodname, params):
+        dumper = partial(json.dumps, cls=JSONEncoder, separators=(',', ':'))
         self.__id += 1
         id_ = self.__id
-        request = json.dumps({
+        if self.__cache and self.__cache.cached(methodname):
+            try:
+                return self.__cache.get(methodname, dumper(params))
+            except KeyError:
+                pass
+        request = dumper({
                 'id': id_,
                 'method': methodname,
                 'params': params,
-                }, cls=JSONEncoder, separators=(',', ':')).encode('utf-8')
+                }).encode('utf-8')
 
         try:
             try:
@@ -278,12 +289,15 @@ class ServerProxy(xmlrpc.client.ServerProxy):
         except Exception:
             self.__transport.close()
             raise
-
         if response['id'] != id_:
             raise ResponseError('Invalid response id (%s) excpected %s' %
                 (response['id'], id_))
         if response.get('error'):
             raise Fault(*response['error'])
+        if self.__cache and response.get('cache'):
+            self.__cache.set(
+                methodname, dumper(params), response['cache'],
+                response['result'])
         return response['result']
 
     def close(self):
@@ -297,8 +311,11 @@ class ServerProxy(xmlrpc.client.ServerProxy):
 
 class ServerPool(object):
     keep_max = 4
+    _cache = None
 
     def __init__(self, host, port, database, *args, **kwargs):
+        if kwargs.get('cache'):
+            self._cache = kwargs['cache'] = _Cache()
         self.ServerProxy = partial(
             ServerProxy, host, port, database, *args, **kwargs)
 
@@ -357,3 +374,41 @@ class ServerPool(object):
         conn = self.getconn()
         yield conn
         self.putconn(conn)
+
+    def clear_cache(self, prefix=None):
+        if self._cache:
+            self._cache.clear(prefix)
+
+
+class _Cache:
+
+    def __init__(self):
+        self.store = defaultdict(dict)
+
+    def cached(self, prefix):
+        return prefix in self.store
+
+    def set(self, prefix, key, expire, value):
+        if isinstance(expire, (int, float)):
+            expire = datetime.timedelta(seconds=expire)
+        if isinstance(expire, datetime.timedelta):
+            expire = datetime.datetime.now() + expire
+        self.store[prefix][key] = (expire, copy.deepcopy(value))
+
+    def get(self, prefix, key):
+        now = datetime.datetime.now()
+        try:
+            expire, value = self.store[prefix][key]
+        except ValueError:
+            raise KeyError
+        if expire < now:
+            self.store.pop(key)
+            raise KeyError
+        logger.info('(cached) %s %s', prefix, key)
+        return copy.deepcopy(value)
+
+    def clear(self, prefix=None):
+        if prefix:
+            self.store[prefix].clear()
+        else:
+            self.store.clear()
